# Copyright (c) 2019-2022, Lawrence Livermore National Security, LLC and
# other Serac Project Developers. See the top-level COPYRIGHT file for details.
#
# SPDX-License-Identifier: (BSD-3-Clause)
#------------------------------------------------------------------------------

cmake_minimum_required(VERSION 3.8 FATAL_ERROR)


####### Expanded from @PACKAGE_INIT@ by configure_package_config_file() #######
####### Any changes to this file will be overwritten by the next CMake run ####
####### The input file was serac-config.cmake.in                            ########

get_filename_component(PACKAGE_PREFIX_DIR "${CMAKE_CURRENT_LIST_DIR}/../" ABSOLUTE)

macro(set_and_check _var _file)
  set(${_var} "${_file}")
  if(NOT EXISTS "${_file}")
    message(FATAL_ERROR "File or directory ${_file} referenced by variable ${_var} does not exist !")
  endif()
endmacro()

macro(check_required_components _NAME)
  foreach(comp ${${_NAME}_FIND_COMPONENTS})
    if(NOT ${_NAME}_${comp}_FOUND)
      if(${_NAME}_FIND_REQUIRED_${comp})
        set(${_NAME}_FOUND FALSE)
      endif()
    endif()
  endforeach()
endmacro()

####################################################################################

if(NOT SERAC_FOUND)

  #----------------------------------------------------------------------------
  # Set version and paths
  #----------------------------------------------------------------------------
  
  set(SERAC_VERSION       "v0.0.1")
  set(SERAC_VERSION_MAJOR "0")
  set(SERAC_VERSION_MINOR "0")
  set(SERAC_VERSION_PATCH "1")
  
  set(SERAC_INSTALL_PREFIX "/usr/local")
  set(SERAC_INCLUDE_DIRS "${SERAC_INSTALL_PREFIX}/include")

  #----------------------------------------------------------------------------
  # Set user configuration options and features
  #----------------------------------------------------------------------------

  set(SERAC_ENABLE_CODEVELOP   OFF)
  set(SERAC_ENABLE_CUDA        OFF)

  set(SERAC_USE_ADIAK          )
  set(SERAC_USE_AXOM           TRUE)
  set(SERAC_USE_CALIPER        )
  set(SERAC_USE_CHAI           )
  set(SERAC_USE_CONDUIT        TRUE)
  set(SERAC_USE_HDF5           TRUE)
  set(SERAC_USE_MFEM           TRUE)
  set(SERAC_USE_MPI            TRUE)
  set(SERAC_USE_PETSC          )
  set(SERAC_USE_RAJA           )
  set(SERAC_USE_TRIBOL         )
  set(SERAC_USE_UMPIRE         )

  set(SERAC_ADIAK_DIR          "")
  set(SERAC_AXOM_DIR           "/usr/local")
  set(SERAC_CALIPER_DIR        "")
  set(SERAC_CHAI_DIR           "")
  set(SERAC_CONDUIT_DIR        "/usr/local")
  set(SERAC_HDF5_DIR           "/usr/local")
  set(SERAC_MFEM_DIR           "/usr/local")
  set(SERAC_PETSC_DIR          "")
  set(SERAC_RAJA_DIR           "")
  set(SERAC_TRIBOL_DIR         "")
  set(SERAC_UMPIRE_DIR         "")

  # Codevelop TPLs are installed alongside Serac
  if(SERAC_ENABLE_CODEVELOP)
    set(SERAC_AXOM_DIR "${SERAC_INSTALL_PREFIX}")
    set(SERAC_MFEM_DIR "${SERAC_INSTALL_PREFIX}")
  endif()

  # Set to real variable unless user overrode it
  foreach(dep AXOM CALIPER CHAI CONDUIT HDF5 MFEM PETSC RAJA TRIBOL UMPIRE)
    if (NOT ${dep}_DIR)
      set(${dep}_DIR "${SERAC_${dep}_DIR}")
    endif()
  endforeach()

  #----------------------------------------------------------------------------
  # Bring in required dependencies for this Serac configuration
  #----------------------------------------------------------------------------
  include(CMakeFindDependencyMacro)

  # MFEM
  if(SERAC_USE_MFEM AND NOT TARGET mfem)
    set(SERAC_MFEM_BUILT_WITH_CMAKE TRUE)
    if(SERAC_MFEM_BUILT_WITH_CMAKE)
      find_dependency(mfem REQUIRED NO_DEFAULT_PATH PATHS "${MFEM_DIR}/lib/cmake/mfem" NAMES MFEM)
      # MFEM include directories are added as the variable but not always added to the target itself
      target_include_directories(mfem SYSTEM INTERFACE "${MFEM_INCLUDE_DIRS}")
    else()
      add_library(mfem INTERFACE IMPORTED GLOBAL)
      target_include_directories(mfem SYSTEM INTERFACE "/usr/local/include;/usr/local/include")
      target_link_libraries(mfem INTERFACE "mfem")
    endif()
  endif()

  # Axom
  # also covers fmt/cli11
  if(SERAC_USE_AXOM)
    find_dependency(Axom REQUIRED NO_DEFAULT_PATH PATHS "${AXOM_DIR}/lib/cmake")
  endif()

  # Conduit
  if(SERAC_USE_CONDUIT)
    find_dependency(Conduit REQUIRED NO_DEFAULT_PATH PATHS "${CONDUIT_DIR}/lib/cmake/conduit")
  endif()

  # HDF5
  if(SERAC_USE_HDF5)
    set(SERAC_HDF5_DIR     "/usr/local")
    # Note: Targets not currently imported
  endif()

  # Tribol
  if(SERAC_USE_TRIBOL)
    find_dependency(tribol REQUIRED NO_DEFAULT_PATH PATHS "${TRIBOL_DIR}/lib/cmake")
  endif()

  # Adiak
  if(SERAC_USE_ADIAK)
    find_dependency(adiak REQUIRED NO_DEFAULT_PATH PATHS "${ADIAK_DIR}")
  endif()

  # Caliper
  if(SERAC_USE_CALIPER)
    # See comment in SetupSeracThirdParty.cmake
    if(SERAC_ENABLE_CUDA)
      if(CMAKE_VERSION VERSION_LESS 3.17)
        message(FATAL_ERROR "Serac+Caliper+CUDA requires CMake > 3.17.")
      else()
        find_package(CUDAToolkit REQUIRED)
      endif() 
    endif()
    find_dependency(caliper REQUIRED NO_DEFAULT_PATH PATHS "${CALIPER_DIR}")
  endif()

  # Petsc
  if(SERAC_USE_PETSC)
    # Note: Targets not currently imported
  endif()

  # Umpire
  if(SERAC_USE_UMPIRE)
    find_dependency(umpire REQUIRED NO_DEFAULT_PATH PATHS "${UMPIRE_DIR}")
  endif()

  # RAJA
  if(SERAC_USE_RAJA)
    find_dependency(raja REQUIRED NO_DEFAULT_PATH PATHS "${RAJA_DIR}")
  endif()

  # CHAI
  if(SERAC_USE_CHAI)
    find_dependency(CHAI REQUIRED NO_DEFAULT_PATH PATHS "${CHAI_DIR}")
  endif()

  #----------------------------------------------------------------------------
  # Include targets exported by CMake
  #----------------------------------------------------------------------------
  get_filename_component(SERAC_CMAKE_CONFIG_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)
  include(${SERAC_CMAKE_CONFIG_DIR}/serac-targets.cmake)

  #----------------------------------------------------------------------------
  # Indicate that Serac is correctly set up
  #----------------------------------------------------------------------------
  set(SERAC_FOUND TRUE)

endif()
